package com.shcd.aistcp.server;

import com.shcd.aistcp.entity.BoatInfoEntity;
import com.shcd.aistcp.service.AisAnalysisService;
import com.shcd.aistcp.util.AisAnalysisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.charset.StandardCharsets;
import java.util.Iterator;

/**
 * 类名:@Name - TcpServer
 * 功能描述:@Function Description<>
 * 日期:@Date - 2025/1/21 14:15
 * 作者:@Created by zrj
 */
@Slf4j
@Component
public class TcpServer {
    @Value("${tcp.server.port}")
    private int port;


    @Autowired
    private AisAnalysisService aisAnalysisService;

    @Autowired
    private AisAnalysisUtil aisAnalysisUtil;

    @PostConstruct
    public void startServer() {
        new Thread(() -> {
            try (ServerSocketChannel serverSocketChannel = ServerSocketChannel.open();
                 Selector selector = Selector.open()) {
                serverSocketChannel.bind(new InetSocketAddress(port));
                serverSocketChannel.configureBlocking(false);
                serverSocketChannel.register(selector, SelectionKey.OP_ACCEPT);
                log.info("服务器启动，监听端口：" + port);
                while (true) {
                    selector.select();
                    Iterator<SelectionKey> keyIterator = selector.selectedKeys().iterator();
                    while (keyIterator.hasNext()) {
                        SelectionKey key = keyIterator.next();
                        keyIterator.remove();
                        if (key.isAcceptable()) {
                            ServerSocketChannel serverChannel = (ServerSocketChannel) key.channel();
                            SocketChannel clientSocketChannel = serverChannel.accept();
                            clientSocketChannel.configureBlocking(false);
                            clientSocketChannel.register(selector, SelectionKey.OP_READ);
                            log.info("客户端已连接：" + clientSocketChannel.getRemoteAddress());
                        } else if (key.isReadable()) {
                            SocketChannel clientSocketChannel = (SocketChannel) key.channel();
                            ByteBuffer buffer = ByteBuffer.allocate(1024);
                            int bytesRead = clientSocketChannel.read(buffer);
                            if (bytesRead == -1) {
                                clientSocketChannel.close();
                                continue;
                            } else if (bytesRead > 0) {
                                buffer.flip();
                                String clientMessage = StandardCharsets.UTF_8.decode(buffer).toString();
                                log.info("解析前 " +clientMessage);
                            // 处理接收到的数据，这里可以添加业务逻辑
                                BoatInfoEntity boatInfoEntity = aisAnalysisUtil.explanation(clientMessage);
                                aisAnalysisService.saveAisAnalysis(boatInfoEntity);
                            }
                        }
                    }
                }
            } catch (IOException e) {
                log.info("服务器启动失败：" + e.getMessage());
            }
        }).start();
    }
}
